#!/usr/bin/env python3
import pygor3 as p3

from  optparse import OptionParser
def main():
    parser = OptionParser()
    parser.add_option("-s", "--species", dest="species", help='Igor species')
    parser.add_option("-c", "--chain", dest="chain", help='Igor chain')
    parser.add_option("-b", "--batch", dest="batch", help='Batchname to identify run. If not set random name is generated')
    parser.add_option("-o", "--output", dest="output", help='filename of csv file to export data')

    (options, args) = parser.parse_args()

    if options.batch == None:
        print("batch is mandatory")
        exit()
    else:
        task = p3.IgorTask.load_from_batchname(str(options.batch))


    if options.species == None:
        print("species is mandatory")
        exit()
    else:
        task.igor_specie = str(options.species)

    if options.chain == None:
        print("species is mandatory")
        exit()
    else:
        task.igor_chain = str(options.chain)

    if options.output == None:
        print("output is mandatory")
        exit()
    else:
        output = options.output

    task.update_batch_filenames()
    task.load_IgorModel()

    strSepChar = ';'

    bs0 = p3.IgorBestScenariosVDJ()
    bsHeaderList = list(eval(str(bs0)).keys())
    strHeader = ";".join(bsHeaderList)
    fln_output_scenarios_names = task.igor_batchname + "_output_scenarios_names.csv"
    ofile = open(fln_output_scenarios_names, "w")
    ofile.write(strHeader + '\n')

    with open(task.igor_fln_output_scenarios, "r") as ifile:
        inputHeader = ifile.readline()
        for line in ifile.readlines():
            # print(line.split(strSepChar), len(line.split(strSepChar)) )
            line = line.replace("\n", "")
            bs = p3.IgorBestScenariosVDJ.load_FromLineBestScenario(line)
            bs.mdlParms = task.mdl.parms
            # print(bs)
            aaa = strSepChar.join(list(map(str, bs.to_dict_names().values())))
            # aaa = strSepChar.join(list(map(str, bs.to_dict_ntsequences().values())))
            ofile.write(aaa+"\n")

    ofile.close()
    print("Exported scenarios in :", fln_output_scenarios_names)

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# TODO: Script to load database given a collection of IGoR files
# 1. genomes
# 2. models
# 3. batch
# 3.1. indexed_sequences
# 3.2. alignments
# 3.3. best scenarios
# 3.4. pgen
# 3.5. coverage


import pygor3 as p3
# from optparse import OptionParser
import argparse
from pathlib import Path

def main():
    parser = argparse.ArgumentParser()

    igor_models = parser.add_argument_group('IGoR default models')

    # Use IGoR default model
    igor_models.add_argument("-s", "--species", dest="species", help='Igor species')
    igor_models.add_argument("-c", "--chain", dest="chain", help='Igor chain')  # , type=str, choices=['TRB', 'TRA'])

    # Or specify directory in IGoR structure.
    parser.add_argument("-M", "--model_path", dest="model_path",
                        help='IGoR model directory path, this path include ref_genomes and model_parms')

    # parser.add_argument("-m", "--set_custom_model", dest="model", nargs=2, help='IGoR model_params.txt')
    parser.add_argument("-g", "--path_ref_genome", dest="path_ref_genome", help='Directory where genome references are stored: genomicDs.fasta,  genomicJs.fasta,  genomicVs.fasta,  J_gene_CDR3_anchors.csv,  V_gene_CDR3_anchors.csv') #, default='./ref_genome')
    # parser.add_argument("--set_genomicVs", dest="genomicVs", help='V gene templates fasta file path')
    # parser.add_argument("--set_genomicDs", dest="genomicDs", help='D gene templates fasta file path')
    # parser.add_argument("--set_genomicJs", dest="genomicJs", help='J gene templates fasta file path')

    parser.add_argument("-w", "--WD", dest="working_directory", help="Path where files gonna be created.", default='./')

    # To load all data files use batchname
    parser.add_argument("-b", "--batch", dest="batch",
                        help='Batchname to identify run. If not set random name is generated', required=True)
    # parser.add_option(dest="path_ref_genome")
    # parser.add_argument("-o", "--output", dest="output", help='filename of csv file to export data')

    args = parser.parse_args()
    print(args)

    # load a task object to get all data in one structure
    task = p3.IgorTask()

    # Now create database to save genome references and also the data
    task.igor_batchname = args.batch  # "sample"  # options.batchname
    task.igor_wd = args.working_directory  # "./test"
    task.update_batch_filenames()
    print(task.igor_fln_db)
    task.create_db()

    task.load_db_from_indexed_sequences()
    task.load_db_from_indexed_cdr3()

    if (args.species is not None) and (args.chain is not None):
        print("species : ", args.species, " chain: ", args.chain )
        try:
            task.run_datadir()
            task.igor_species = args.species
            task.igor_chain = args.chain
            igor_model_path = task.igor_models_root_path + task.igor_species + "/" + p3.igor_option_path_dict[task.igor_chain] + "/"
            # igor_model_path = task.igor_models_root_path + task.igor_species + "/" \
            #                 + p3.igor_option_path_dict[task.igor_chain] + "/"
            args.model_path = igor_model_path
            print(args.model_path)
        except Exception as e:
            print("Default model not found!")
            print(e)

    # IF MODEL_PATH PROVIDED THEN
    # task.igor_model_dir_path = args.model_path
    if (args.model_path is not None):
        try:
            task.update_model_filenames(args.model_path)
            # task.igor_model_parms_file = task.igor_model_dir_path+"/models/model_parms.txt"
            # task.igor_model_marginals_file = task.igor_model_dir_path+"/models/model_marginals.txt"
            # task.igor_path_ref_genome = task.igor_model_dir_path+"/ref_genome/"
            # task.load_IgorModel()
            args.path_ref_genome = task.igor_path_ref_genome
            task.load_db_from_models()
        except Exception as e:
            print("Couldn't load models to database")
            print(e)

    # IF GENOME PATH PROVIDED
    if args.path_ref_genome is not None:
        try:
            task.igor_path_ref_genome = args.path_ref_genome  # "/home/alfaceor/Dropbox/PosDoc/IGoR/dev/MyGithub/pygor3/demo/thi/genomics_repseqio_F"
            task.load_IgorRefGenome()
            # print(task.igor_fln_indexed_sequences)
            # print(task.genomes.df_genomicVs) # is where all this data is collected
            # print(task.genomes.df_genomicDs)
            # print(task.genomes.df_genomicJs)

            task.load_db_from_genomes()
        except Exception as e:
            print("Couldn't load genome templates to database")
            print("ERROR: ", e)

    task.load_db_from_alignments()

    task.load_db_from_bestscenarios()

    task.load_db_from_pgen()

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# modify genomic templates and anchors to short_names
import pygor3 as p3
from Bio import SeqIO
import pandas as pd

def main():
    from optparse import OptionParser
    parser = OptionParser(usage="usage: %prog [options] ")
    parser.add_option("-t", "--gene_template",   dest="gene_template",   help="fasta file with gene templates")
    parser.add_option("-a", "--gene_anchors",   dest="gene_anchors",   help="csv file with gene anchors")

    (options, args) = parser.parse_args()

    flnGenome = options.gene_template
    flnAnchors = options.gene_anchors

    records = p3.imgt.load_records_from_fasta(flnGenome)
    new_records = list()
    for record in records:
        record.description = p3.genLabel(record.description)
        record.id = record.description
        new_records.append(record)
    p3.imgt.save_records2fasta(new_records, flnGenome+"_short")

    try:
        df = pd.read_csv(flnAnchors, sep=';')
        print(df['gene'])
        df['gene'] = df['gene'].map(p3.genLabel)
        print(df['gene'])
        df.to_csv(flnAnchors+"_short", sep=';', index=False)
    except Exception as e:
        print(e)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import pygor3 as p3

from  optparse import OptionParser
def main():
    parser = OptionParser()
    parser.add_option("-s", "--species", dest="species", help='Igor species')
    parser.add_option("-c", "--chain", dest="chain", help='Igor chain')
    parser.add_option("-b", "--batch", dest="batch", help='Batchname to identify run. If not set random name is generated')
    parser.add_option("-o", "--output", dest="output", help='filename of csv file to export data')

    (options, args) = parser.parse_args()

    igor_specie="human"
    igor_chain="tcr_beta"
    mdl0 = p3.IgorModel.load_default(igor_specie, igor_chain)
    #mdl0.parms.plot_Graph()
    #print( p3.Igor_vj_dict_default)
    #print( p3.IgorRec_Event_default_dict.keys() )
    # load Model_Parms

    mdl_parms = p3.IgorModel_Parms()
    aaa = p3.IgorRec_Event_default_dict['v_choice']
    print(aaa)
    event = p3.IgorRec_Event.from_dict(aaa)
    mdl_parms.Event_list.append(event)

    mdl_parms.Event_list.append(p3.IgorRec_Event.from_dict(p3.IgorRec_Event_default_dict['v_choice']))
    print(mdl_parms.Event_list) #load_events_from_dict({})
    for ev in mdl_parms.Event_list:
        print(ev)
    # evento = mdl_parms.Event_list[0]
    # print(evento)

    """
    vj_dict_default = {
        'v_choice': {},
        'j_choice': {'v_choice'},
        'd_gene': {'v_choice', 'j_choice'},
        'v_3_del': {'v_choice'},
        'd_5_del': {'d_gene'},
        'd_3_del': {'d_gene', 'd_5_del'},
        'j_5_del': {'j_choice'},
        'vd_ins': {},
        'vd_dinucl': {},
        'dj_ins': {},
        'dj_dinucl': {}
    }

    vj_dict_parms = {
        'v_choice': {},
        'j_choice': {'v_choice'},
        'd_gene': {'v_choice', 'j_choice'},
        'v_3_del': {'v_choice'},
        'd_5_del': {'d_gene'},
        'd_3_del': {'d_gene', 'd_5_del'},
        'j_5_del': {'j_choice'},
        'vd_ins': {},
        'vd_dinucl': {},
        'dj_ins': {},
        'dj_dinucl': {}
    }

    # Given a dictionary create Bayes network
    mdl1_parms = p3.IgorModel_Parms.from_network_dict(vj_dict_default)
    # this should generate a minimal default model.
    # And what I mean with that? Event_list without realizations
    print(mdl1_parms.Event_list[0])
    # Now add the realizations to the in Parms class
    mdl1_parms.Event_dict['v_choice']

    # So the idea is that
    pathGenomic = p3.rcParams['paths.igor_models']+ "/human/tcr_beta/"+"ref_genome/"

    vj_dict_parms = {
        'v_choice': {"filename" : pathGenomic+"genomicVs.fasta"},
        'j_choice': {"filename" : pathGenomic+"genomicJs.fasta"},
        'd_gene': {"filename" : pathGenomic+"genomicDs.fasta"},
        'v_3_del': {"limits" :(-4,20)},
        'd_5_del': {"limits" : (-4,20)},
        'd_3_del': {"limits" : (-4,20)},
        'j_5_del': {"limits" : (-4,20)},
        'vd_ins': {"limits" : (0,24)},
        'vd_dinucl': {},
        'dj_ins': {"limits" : (0,24)},
        'dj_dinucl': {}
    }



    mdl1_parms.load_from_dict()
    mdl1_parms.load_GeneChoice_realizations_by_nickname('v_choice', pathGenomic+"genomicVs.fasta")
    mdl1_parms.load_GeneChoice_realizations_by_nickname('j_choice', pathGenomic+"genomicJs.fasta")
    mdl1_parms.load_GeneChoice_realizations_by_nickname('d_gene', pathGenomic+"genomicDs.fasta")
    mdl1_parms.load_Deletion_realizations_by_nickname('v_3_del')
    mdl1_parms.load_Deletion_realizations_by_nickname('d_5_del')
    mdl1_parms.load_Deletion_realizations_by_nickname('d_3_del')
    mdl1_parms.load_Deletion_realizations_by_nickname('j_5_del')
    mdl1_parms.load_Insertion_realizations_by_nickname('vd_ins')
    mdl1_parms.load_Insertion_realizations_by_nickname('dj_ins')
    mdl1_parms.load_DinucMarkov_realizations_by_nickname('vd_dinucl')
    mdl1_parms.load_DinucMarkov_realizations_by_nickname('dj_dinucl')
    """


if __name__ == "__main__":
    main()


#!/usr/bin/env python3
import pygor3 as p3
import pandas as pd

from optparse import OptionParser

def main():
    parser = OptionParser()

    (options, args) = parser.parse_args()
    # fln_model_parms = args[0]
    # fln_model_marginals = args[1]
    # mdl = p3.IgorModel(model_parms_file=fln_model_parms, model_marginals_file=fln_model_marginals)
    mdl = p3.IgorModel.load_default('human', 'tcr_beta')

    strEvent='v_choice'
    print(mdl.xdata[strEvent])
    aaa = mdl.xdata[strEvent].plot()
    print(type(aaa))
    import matplotlib.pyplot as plt
    plt.show()


    """
    # for strEvent in mdl.xdata.keys():
    #     mdl.xdata[strEvent]
    strEvent = 'v_choice'
    print(mdl.xdata[strEvent].dims)
    print('*'*50)
    import matplotlib.pyplot as plt

    fig, ax = plt.subplots()
    strEvent = 'v_choice'
    df = mdl.xdata[strEvent].to_dataframe(name='P')
    df.plot.bar(x='lbl__' + strEvent, y='P', rot=90, ax=ax)
    fig.tight_layout()
    fig.savefig(strEvent + ".pdf")

    fig, ax = plt.subplots()
    strEvent = 'j_choice'
    da = mdl.xdata[strEvent]
    da = da.sum(dim='v_choice')
    P_J = da
    df = da.to_dataframe(name='P')

    df.plot.bar(x='lbl__' + strEvent, y='P', rot=90, ax=ax)
    fig.tight_layout()
    fig.savefig(strEvent + ".pdf")
    
    """

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import pygor3 as p3
import argparse

def get_pairwise_prob(mdl, event_nickname1, event_nickname2):
    # create an xarray matrix with event_nickname1 and event_nickname2
    da = mdl.get_zero_xarray_from_list([event_nickname1, event_nickname2])
    # print("da = ", da)
    import xarray as xr

    def tmp_funct(bs:p3.IgorScenario):
        ev_dict_ids = {event_nickname1: bs['id_'+event_nickname1], event_nickname2: bs['id_'+event_nickname2]}
        darr = xr.zeros_like(da)
        darr.loc[ev_dict_ids] = 1
        return darr

    return tmp_funct

def main():
    parser = argparse.ArgumentParser()
    igor_models = parser.add_argument_group('IGoR default models')

    # # Use IGoR default model
    # igor_models.add_argument("-s", "--species", dest="species", help='Igor species', default="human")
    # igor_models.add_argument("-c", "--chain", dest="chain", help='Igor chain', default="tcr_beta")  # , type=str, choices=['TRB', 'TRA'])

    parser.add_argument("-D", "--database", dest="database", help="Igor database created with database script.", default="Ajam.db")
    parser.add_argument("--event_pair", dest="event_pair", help="Events nickname", nargs=2, default=['v_choice', 'j_choice'])

    igor_models = parser.add_argument_group('IGoR envent_pairdefault models')

    args = parser.parse_args()

    print(args)
    print(args.event_pair)
    str_event_nickname1 = args.event_pair[0]
    str_event_nickname2 = args.event_pair[1]

    # Create an IgorModel
    # mdl = p3.IgorModel.load_default(args.species, args.chain)
    db = p3.IgorSqliteDB.create_db(args.database)
    mdl = db.get_IgorModel()
    func_pairwise = get_pairwise_prob(mdl, str_event_nickname1, str_event_nickname2)
    seq_index = 3
    average = db.calc_IgorBestScenarios_average_of(func_pairwise) #, indices_list=[seq_index])
    bs_list = db.get_IgorBestScenarios_By_seq_index_IgorModel(seq_index, mdl)
    bs = bs_list[0]
    bs_realiz = mdl.parms.realiz_dict_from_scenario(bs)
    print(bs_realiz )
    # print(bs_realiz['v_choice'].to_dict())
    print( bs_realiz['mismatches'].value )
    return 0

    print("=" * 50)
    bs_records = db.fetch_IgorBestScenarios_By_seq_index(seq_index)
    # import pandas as pd
    # bs_df = pd.DataFrame.from_records(bs_records)
    bs_df = db.get_IgorBestScenariosDataframe_By_seq_index(seq_index)

    print(bs_df[bs_df.id_v_choice == 59]['scenario_proba_cond_seq'].sum() )
    print(bs_df[bs_df.id_v_choice == 60]['scenario_proba_cond_seq'].sum())

    import matplotlib.pyplot as plt
    fig, ax = plt.subplots(figsize=(10, 15))
    aaa = average.plot(ax=ax, cmap='gnuplot2_r') #, vmin=0, vmax=1.0)
    print(type(aaa))

    str_title = "$P($" + str_event_nickname1 + ", " + str_event_nickname2+"$)$"

    ax.set_ylabel(str_event_nickname1.replace("_", " "))
    ax.set_xlabel(str_event_nickname2.replace("_", " "))

    import numpy as np
    v_genlabel = np.vectorize(p3.genLabel)

    XX = average[str_event_nickname2].values
    lbl_XX = average['lbl__' + str_event_nickname2].values

    YY = average[str_event_nickname1].values
    lbl_YY = average['lbl__' + str_event_nickname1].values

    ax.set_xticks(XX)
    ax.set_xticklabels(v_genlabel(lbl_XX), rotation=90)

    ax.set_yticks(YY)
    ax.set_yticklabels(v_genlabel(lbl_YY))

    fig.savefig('figura.pdf')
    # return ax

    plt.show()

    return 0

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Aug 12 18:58:54 2019

@author: alfaceor
"""
import pygor3.imgt as imgt

def main():
    from optparse import OptionParser
    parser = OptionParser(usage="usage: %prog [options] ")
    parser.add_option("-t", "--type",   dest="type",   help="VJ or VDJ")
    parser.add_option("-g", "--gene",   dest="gene",   help="Type of gene like TRB")
    parser.add_option("-s", "--specie", dest="specie", help="Type of specie")

    (options, args) = parser.parse_args()

    #options.type   = "VDJ"
    #options.gene   = "TRB"
    #options.specie = "Mus+musculus"


    if options.type == "VDJ":
        if options.gene == None:
            print("Gene option is missing: --gene TRB")
        else:
            if options.specie == None:
                print("Specie option is missing: --specie Mus+musculus")
            else:
                imgt.makeDirectories(options.gene, options.specie)
                imgt.write_V_TemplateFiles(options.gene, options.specie)
                imgt.write_D_TemplateFiles(options.gene, options.specie)
                imgt.write_J_TemplateFiles(options.gene, options.specie)
                # Now write the model. by using the template files.
                imgt.writeModelVDJ_Parms(options.gene, options.specie)
                imgt.writeModelVDJ_Marginals(options.gene, options.specie)

    elif options.type == "VJ":
        if options.gene == None:
            print("Gene option is missing: --gene TRB")
        else:
            if options.specie == None:
                print("Specie option is missing: --specie Mus+musculus")
            else:
                imgt.makeDirectories(options.gene, options.specie)
                imgt.write_V_TemplateFiles(options.gene, options.specie)
                imgt.write_J_TemplateFiles(options.gene, options.specie)
                imgt.writeModelVJ_parms(options.gene, options.specie)
                imgt.writeModelVJ_Marginals(options.gene, options.specie)
    else:
        print("Too bad you need to specify a type : --type VDJ")


if __name__ == "__main__":
    main()


#!/usr/bin/env python3
"""
Export Igor Model to csv files.
"""

import pygor3 as p3
import pandas as pd


from optparse import OptionParser

def main():
    task = p3.IgorTask()
    parser = OptionParser()
    parser.add_option("-s", "--species", dest="species", help='Igor species')
    parser.add_option("-c", "--chain", dest="chain", help='Igor chain')
    parser.add_option("-o", "--output", dest="output", help='filename of csv file to export data')

    parser.add_option("-p", "--model_params", dest="model_params", help='IGoR model_params.txt')
    parser.add_option("-m", "--model_marginals", dest="model_marginals", help='IGoR model_marginals.txt')


    (options, args) = parser.parse_args()

    if options.output == None:
        print("output option is mandatory")
        exit()
    else:
        flnPrefix = options.output

    Q_species_chain = ( not (options.species == None) and not(options.chain == None))
    Q_model_files = ( not (options.model_params == None) and not( options.model_marginals == None))
    if ( Q_species_chain or Q_model_files):

        import numpy as np
        import matplotlib.pyplot as plt

        if Q_species_chain :
            task.igor_species = str(options.species)
            task.igor_chain = str(options.chain)
            task.load_IgorModel()

            for event_nickname in task.mdl.Pmarginal.keys():
                fig, ax = plt.subplots()
                # df = task.mdl.Pmarginal[event_nickname].to_dataframe(name=event_nickname)
                task.mdl.plot_Event_Marginal(event_nickname, ax=ax)
                fig.tight_layout()
                flnOutput = flnPrefix+"_"+event_nickname+".pdf"
                fig.savefig(flnOutput)
                print("***** Marginal plot of ", event_nickname, " in ", flnOutput)
                # if task.mdl.parms.get_Event(event_nickname).event_type == 'Dinucl':
                #     df.plot.bar(y=event_nickname, x='lbl__'+event_nickname, ax=ax)
                #     fig.savefig("PM_"+event_nickname+".pdf")


        if Q_model_files:
            task.igor_model_parms_file = options.model_params
            task.igor_model_marginals_file = options.model_marginals
            task.load_IgorModel()

            for event_nickname in task.mdl.Pmarginal.keys():
                fig, ax = plt.subplots()
                # df = task.mdl.Pmarginal[event_nickname].to_dataframe(name=event_nickname)
                task.mdl.plot_Event_Marginal(event_nickname, ax=ax)
                fig.tight_layout()
                flnOutput = flnPrefix+"_"+event_nickname+".pdf"
                fig.savefig(flnOutput)
                print("***** Marginal plot of ", event_nickname, " in ", flnOutput)

            task.mdl.export_csv(options.output)

    else:
        if not Q_species_chain :
            print("species and chain is mandatory")
            exit()

        if not Q_model_files :
            print("models files are mandatory")
            exit()

    # if options.chain == None:
    #     print("species is mandatory")
    #     exit()
    # else:
    #     task.igor_chain = str(options.chain)
    #
    # if options.output == None:
    #     print("output is mandatory")
    #     exit()
    # else:
    #     output = options.output
    #
    # if options.batch == None:
    #     print("Batchname not set. Random batchname will be assing.")

    # if len(args) > 0 and len(args) <2:
    #   filename = str(args[0])
    #   task.igor_read_seqs = filename
    # else:
    #   print("Supply a correct filename")
    #   exit()

    # task.run_evaluate()
    # task.update_batch_filenames()

    #
    # for strEvent in task.mdl.xdata.keys():
    #     da = task.mdl.xdata[strEvent]
    #     dependencias = list(task.mdl.xdata[strEvent].dims)
    #     print("********", dependencias, strEvent)
    #     dependencias.remove(strEvent)
    #     dependencias_dim = [task.mdl.xdata[strEvent][dep].shape[0] for dep in dependencias]
    #     print(strEvent, da.dims, dependencias_dim, dependencias)
    #     if len(dependencias) == 0 :
    #         lbl_file = "P__"+strEvent
    #         print(lbl_file)
    #     elif len(dependencias) == 1:
    #         lbl_file = "P__" + strEvent +"__g__"+dependencias[0]
    #         print(lbl_file)
    #         df = pd.DataFrame(data=da.values, index=da['lbl__' + dependencias[0]].values,
    #                           columns=da['lbl__' + strEvent].values)
    #         #print(df)
    #     elif len(dependencias) == 2 :
    #         lbl_file = "P__" + strEvent + "__g__" + dependencias[0]+"__"+ dependencias[1]
    #         print(lbl_file)
    #     elif len(dependencias) == 3:
    #         lbl_file = "P__" + strEvent + "__g__" + dependencias[0] + "__" + dependencias[1]+ "__" + dependencias[2]
    #         print(lbl_file)
    #     else:
    #         print("OHHHHHHHH")
    #     #df = pd.DataFrame(data=da.values, index=da['lbl__' + 'v_choice'].values, columns=da['lbl__' + 'j_choice'].values)
    #
    # task.run_clean_batch()

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import pygor3 as p3


def get_V_anchor_index_in_seq_from_gene(align:p3.IgorAlignment_data):
    ins_size = len(align.insertions)
    dels_size = len(align.deletions)
    dict_gene_anchor_index = dict()
    # TODO: COUNT number of insertions before cdr3_v_gene_anch



# TODO: THERE IS A WAY TO GET
def generate_str_fasta(indexed_sequence, list_vdj_alignments:dict):
    """ Given an Sequence index and the corresponding alignments vj/ vdj
    return a string with considering only offset"""

    indexed_sequence.sequence = indexed_sequence.sequence.lower()
    # add mismatches in sequence.
    s = list(indexed_sequence.sequence)
    for key_align in list_vdj_alignments.keys():
        for pos_mis in list_vdj_alignments[key_align].mismatches:
            s[pos_mis] = s[pos_mis].upper()
    indexed_sequence.sequence = "".join(s)

    str_fasta = ""
    min_offset_key = min(list_vdj_alignments.keys(), key=lambda x: list_vdj_alignments[x].offset)  # .offset
    min_offset = list_vdj_alignments[min_offset_key].offset
    min_offset = min(indexed_sequence.offset, min_offset)

    delta_offset = indexed_sequence.offset - min_offset
    str_prefix = '-' * (delta_offset)
    str_fasta_sequence = str_prefix + indexed_sequence.sequence
    print(str_fasta_sequence)
    str_fasta = str_fasta + "> " + str(indexed_sequence.seq_index) + "\n"
    str_fasta = str_fasta + str_fasta_sequence + "\n"
    for key in list_vdj_alignments.keys():
        list_vdj_alignments[key].strGene_seq = list_vdj_alignments[key].strGene_seq.lower()
        delta_offset = list_vdj_alignments[key].offset - min_offset
        str_prefix = '-' * (delta_offset)
        str_fasta_sequence = str_prefix + list_vdj_alignments[key].strGene_seq
        print(str_fasta_sequence)
        str_fasta = str_fasta + "> " + key + ", " + list_vdj_alignments[key].strGene_name + "\n"
        str_fasta = str_fasta + str_fasta_sequence + "\n"
        offset_5_p = list_vdj_alignments[key].offset_5_p - min_offset
        offset_3_p = list_vdj_alignments[key].offset_3_p - min_offset
        print("delta_offset : ", delta_offset)
        print("offset_5_p : ", list_vdj_alignments[key].offset_5_p, offset_5_p)
        print("offset_3_p : ", list_vdj_alignments[key].offset_3_p, offset_3_p)
        str_prefix_2 = '-' * (offset_5_p+1)
        str_fasta_sequence2 = str_prefix_2 + str_fasta_sequence[offset_5_p+1:offset_3_p+1]

        str_fasta = str_fasta + "> " + list_vdj_alignments[key].strGene_name + ", score : "+str(list_vdj_alignments[key].score) + "\n"
        str_fasta = str_fasta + str_fasta_sequence2 + "\n"

        # TODO ADD MISMATCHES
        align = list_vdj_alignments[key]
        # align mismatches are in indexed sequence reference I need to convert it to gene reference given the alignment
        # given the align.offset
        # pos_in_gene  = pos_in_seq - align.offset
        # pos_in_gene = cdr3 - align.offset

    return str_fasta

    # mixcr :
    # targetSequences	targetQualities	allVHitsWithScore
    # allDHitsWithScore	allJHitsWithScore	allCHitsWithScore	allVAlignments	allDAlignments
    # allJAlignments	allCAlignments	nSeqFR1	minQualFR1	nSeqCDR1	minQualCDR1	nSeqFR2	minQualFR2	nSeqCDR2	minQualCDR2	nSeqFR3	minQualFR3	nSeqCDR3	minQualCDR3	nSeqFR4	minQualFR4	aaSeqFR1	aaSeqCDR1	aaSeqFR2	aaSeqCDR2	aaSeqFR3	aaSeqCDR3	aaSeqFR4	refPoints

    # AIRR :
    # sequence_id (req)
    # sequence (req)
    # rev_comp (req) //false by default # FIXME: for now
    # productive (req) // false by default # FIXME: a naive alignmnet of productive is possible.
    # locus (chain type) // eg. TCR IG
    # v_call
    # d_call
    # d2_call // FIXME: not gonna use this by the moment.
    # j_call
    # sequence alignment // FIXME: Aligned portion of query sequence. IMGT-gaps check the best way to do this
    # germline_alignment // FIXME: Assembled, aligned, full-length inferred germline sequence spanning the same region as the sequence_alignment field (typically the V(D)J region) and including the same set of corrections and spacers (if any)
    # junction
    # junction_aa // Our CDR3
    # np1 (nts btwn V and D or btwn V and J  // vd_ins  or vj_ins
    # np2 (nts btwn first D and J gene or btwn first D and second D // dj_ins or d1d2_ins
    # np3 (nts btwn second D and J gene
    # v_score
    # v_cigar
    # d_score
    # d_cigar
    # j_score
    # j_cigar
    # junction_length


import argparse

def main():

    parser = argparse.ArgumentParser()
    parser.add_argument("-D", "--database", dest="database", help="Igor database created with database script.")
    #parser.add_argument("-b", "--batch", dest="batch", help='Batchname to identify run. If not set random name is generated')
    parser.add_argument("-G", "--gene", dest="gene", choices=['V', 'D', 'J'], help='Gene to get multiple alignments i.e V, D, or J.')
    parser.add_argument("-o", "--output", dest="output", help='filename of csv file to export data')
    parser.add_argument("-i", "--seq_index", dest="seq_index", type=int, default=0, help='Seq index in database/<batch>_indexed_sequences.csv')
    # parser.add_argument("-g", "--path_ref_genome", dest="path_ref_genome", help='Directory where genome references are stored: genomicDs.fasta,  genomicJs.fasta,  genomicVs.fasta,  J_gene_CDR3_anchors.csv,  V_gene_CDR3_anchors.csv', default='./ref_genome')
    # parser.add_argument("-w", "--WD", dest="working_directory", help="Path where files gonna be created.", default='./')
    # parser.add_argument("")

    args = parser.parse_args()

    db = p3.IgorSqliteDB()
    # db.flnIgorDB = task.igor_wd+"/"+task.igor_batchname+".db"
    db.flnIgorDB = args.database
    db.connect_db()

    seq_index = args.seq_index
    indexed_sequence = db.get_IgorIndexedSeq_By_seq_index(seq_index)
    indexed_sequence.offset = 0
    #
    # best_v_align_data = db.get_best_IgorAlignment_data_By_seq_index('V', indexed_sequence.seq_index)
    # best_j_align_data = db.get_best_IgorAlignment_data_By_seq_index('J', indexed_sequence.seq_index)
    align_data_list = db.get_IgorAlignment_data_list_By_seq_index(args.gene, indexed_sequence.seq_index)

    align_dict = {align.strGene_name : align for align in align_data_list}
    # print(align_dict)

    if args.output is None:
        batchname = db.flnIgorDB.split(".db")[0]
        fln_output = batchname + '__' + str(indexed_sequence.seq_index) + '_'+ args.gene +'_na.fasta'
        # fln_output = args.database.split(".db")[0]+"_na.csv"
    else:
        fln_output = args.output

    ofile = open(fln_output, 'w')
    str_fasta = generate_str_fasta(indexed_sequence, align_dict)
    ofile.write(str_fasta)
    ofile.close()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import pygor3 as p3

from  optparse import OptionParser
def main():
    parser = OptionParser()
    parser.add_option("-s", "--species", dest="species", help='Igor species')
    parser.add_option("-c", "--chain", dest="chain", help='Igor chain')
    parser.add_option("-b", "--batch", dest="batch", help='Batchname to identify run. If not set random name is generated')
    parser.add_option("-o", "--output", dest="output", help='filename of csv file to export data')

    (options, args) = parser.parse_args()

    if options.batch == None:
        print("batch is mandatory")
        exit()
    else:
        task = p3.IgorTask.load_from_batchname(str(options.batch))


    if options.species == None:
        print("species is mandatory")
        exit()
    else:
        task.igor_specie = str(options.species)

    if options.chain == None:
        print("species is mandatory")
        exit()
    else:
        task.igor_chain = str(options.chain)

    if options.output == None:
        print("output is mandatory")
        exit()
    else:
        output = options.output

    task.update_batch_filenames()
    task.load_IgorModel()

    strSepChar = ';'

    bs0 = p3.IgorBestScenariosVDJ()
    bsHeaderList = list(eval(str(bs0)).keys())
    strHeader = ";".join(bsHeaderList)
    fln_output_scenarios_names = task.igor_batchname + "_output_scenarios_names.csv"
    ofile = open(fln_output_scenarios_names, "w")
    ofile.write(strHeader + '\n')

    ii = 0
    chosen_V = 0
    chosen_D = 1
    chosen_J = 2
    # Keep the
    normalization_factor = 0
    with open(task.igor_fln_output_scenarios, "r") as ifile:
        inputHeader = ifile.readline()
        # Read line by line but make a group of the same seq_index
        # FIXME: read_seq_index_from_bs(ii)
        prev_seq_index = 99
        list_of_bs = list()
        for line in ifile.readlines():
            # print(line.split(strSepChar), len(line.split(strSepChar)) )
            line = line.replace("\n", "")
            bs = p3.IgorBestScenariosVDJ.load_FromLineBestScenario(line)
            bs.mdlParms = task.mdl.parms
            if bs.seq_index == prev_seq_index:
                list_of_bs.append(bs)
                normalization_factor = normalization_factor + bs.scenario_proba_cond_seq
                aaa = strSepChar.join(list(map(str, bs.to_dict_ntsequences().values())))
                ofile.write(aaa+"\n")
            else:
                # Normalize the probabilities
                if(bs.id_v_choice == chosen_V) :
                    # count this prob scenario.
                    chosen_V_prob
                normalization_factor = 0



    ofile.close()
    print("Exported scenarios in :", fln_output_scenarios_names)

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import pygor3 as p3
import argparse

def generate_event_delta_Kronecker(event_nickname, event_id):
    def tmp_funct(bs:p3.IgorScenario):
        if bs[event_nickname] == event_id:
            return 1
        else:
            return 0
    return tmp_funct

def get_pairwise_prob(mdl, event_nickname1, event_nickname2):
    # create an xarray matrix with event_nickname1 and event_nickname2
    # da = xr.
    da = mdl.get_zero_xarray_from_list([event_nickname1, event_nickname2])

    def tmp_funct(bs:p3.IgorScenario):
        ev_dict_ids = {event_nickname1: bs['id_'+event_nickname1], event_nickname2: bs['id_'+event_nickname2]}
        da.loc[ev_dict_ids] = 1
        return da

    return tmp_funct


def calc_average(db:p3.IgorSqliteDB, observable_function):
    for sigma in db.fetch_IgorIndexedSeq_indexes():
        print(sigma, len(db.fetch_IgorIndexedSeq_indexes()))

# from  optparse import OptionParser
def main():
    #parser = OptionParser()
    #parser.add_option("-s", "--species", dest="species", help='Igor species')
    #parser.add_option("-c", "--chain", dest="chain", help='Igor chain')
    #parser.add_option("-b", "--batch", dest="batch", help='Batchname to identify run. If not set random name is generated')
    #parser.add_option("-o", "--output", dest="output", help='filename of csv file to export data')

    parser = argparse.ArgumentParser()
    igor_models = parser.add_argument_group('IGoR default models')

    # # Use IGoR default model
    # igor_models.add_argument("-s", "--species", dest="species", help='Igor species', default="human")
    # igor_models.add_argument("-c", "--chain", dest="chain", help='Igor chain', default="tcr_beta")  # , type=str, choices=['TRB', 'TRA'])

    parser.add_argument("-D", "--database", dest="database", help="Igor database created with database script.", default="Ajam.db")
    parser.add_argument("--event_pair", dest="event_pair", help="Events nickname", nargs=2, default=['v_choice', 'j_choice'])

    igor_models = parser.add_argument_group('IGoR envent_pairdefault models')
    mdl = p3.IgorModel.load_default("mouse", "tcr_beta")
    args = parser.parse_args()

    fln_database = args.database
    db = p3.IgorSqliteDB.create_db(fln_database)
    records = db.get_list_of_tables_with_name('Igor%')
    print(records)
    db.delete_IgorModel_Tables()
    records = db.get_list_of_tables_with_name('Igor%')
    print(records)


    return 0

    # Which variable I want to eliminate
    strEvent = 'v_3_del'
    sorted_events = mdl.parms.get_Event_list_sorted()
    sorted_events_to_marginalize = [event for event in sorted_events if not event.event_type == "DinucMarkov"]
    sorted_events_to_marginalize_without_VE = [ event for event in sorted_events_to_marginalize if not event.nickname == strEvent]

    # Start eliminating events
    factors = mdl.VE_get_Pmarginals_initial_factors()
    for event_to_eliminate_VE in sorted_events_to_marginalize_without_VE:
        factors = mdl.VE_get_factors_by_sum_out_variable(event_to_eliminate_VE.nickname, factors)

    Pmarginal = 1
    # Now I want to print each factor
    for factor in factors:
        Pmarginal = Pmarginal * factor
    print(Pmarginal)

    # event_to_eliminate_VE = sorted_events_to_marginalize_without_VE.pop(0)
    # factors = mdl.VE_get_factors_by_sum_out_variable(event_to_eliminate_VE.nickname, factors)
    # print(len(factors))
    #
    # event_to_eliminate_VE = sorted_events_to_marginalize_without_VE.pop(0)
    # factors = mdl.VE_get_factors_by_sum_out_variable(event_to_eliminate_VE.nickname, factors)
    # print(len(factors))
    #
    # event_to_eliminate_VE = sorted_events_to_marginalize_without_VE.pop(0)
    # factors = mdl.VE_get_factors_by_sum_out_variable(event_to_eliminate_VE.nickname, factors)
    # print(len(factors))
    # # print(factors[-1])
    # # print( mdl.xdata['v_choice'] )
    # #
    # # print( mdl.xdata['v_choice']*factors[-1] )
    #
    # event_to_eliminate_VE = sorted_events_to_marginalize_without_VE.pop(0)
    # factors = mdl.VE_get_factors_by_sum_out_variable(event_to_eliminate_VE.nickname, factors)
    # print(len(factors))
    #
    # event_to_eliminate_VE = sorted_events_to_marginalize_without_VE.pop(0)
    # factors = mdl.VE_get_factors_by_sum_out_variable(event_to_eliminate_VE.nickname, factors)
    # print(len(factors))
    #
    # event_to_eliminate_VE = sorted_events_to_marginalize_without_VE.pop(0)
    # factors = mdl.VE_get_factors_by_sum_out_variable(event_to_eliminate_VE.nickname, factors)
    # print(len(factors))
    #
    # event_to_eliminate_VE = sorted_events_to_marginalize_without_VE.pop(0)
    # factors = mdl.VE_get_factors_by_sum_out_variable(event_to_eliminate_VE.nickname, factors)
    # print(len(factors))
    #
    # event_to_eliminate_VE = sorted_events_to_marginalize_without_VE.pop(0)
    # factors = mdl.VE_get_factors_by_sum_out_variable(event_to_eliminate_VE.nickname, factors)
    # print(len(factors))
    #
    # Pmarginal = 1
    # if len(sorted_events_to_marginalize_without_VE) == 0:
    #     # Now I want to print each factor
    #     for factor in factors:
    #         Pmarginal = Pmarginal*factor
    #     print(Pmarginal)
    #




    # print(mdl.Pmarginal['j_choice'])
    # da = mdl.xdata['j_choice']*mdl.xdata['v_choice']
    # print(da.sum('v_choice'))

    return 0
    sorted_events = mdl.parms.get_Event_list_sorted()
    sorted_events_to_marginalize = [event for event in sorted_events if not event.event_type == "DinucMarkov"]
    strEvent = 'j_choice'
    event_to_marginalize = mdl.parms.get_Event(strEvent)
    event_to_eliminate_ordered_list = [event for event in sorted_events_to_marginalize if not
                                       event.nickname == event_to_marginalize.nickname]
    factors = mdl.VE_get_Pmarginals_initial_factors()
    print("event_to_marginalize.nickname : ", event_to_marginalize.nickname)
    for event_to_eliminate in event_to_eliminate_ordered_list:
        print("event_to_eliminate : ", event_to_eliminate.nickname)
        mdl.VE_get_factors_by_sum_out_variable(event_to_eliminate.nickname)
        print(mdl.factors)

    # mdl.Pmarginal[event_to_marginalize.nickname] = mdl.factors
    # print("-------- FACTORS")
    # print(mdl.factors)
    # print("====================")
    # print(mdl.xdata[strEvent])
    # print("--------------------")
    # print( mdl.Pmarginal[strEvent] )
    # print("////////////////////")
    # # At the end only one factor should survive

    return 0

    args = parser.parse_args()
    print(args)
    print(args.event_pair)
    str_event_nickname1 = args.event_pair[0]
    str_event_nickname2 = args.event_pair[1]

    # Create an IgorModel
    #mdl = p3.IgorModel.load_default(args.species, args.chain)
    db = p3.IgorSqliteDB.create_db(args.database)
    mdl = db.get_IgorModel()
    print(mdl['j_choice'])
    # TODO: ahora que tenemos el nombre de
    #  las columnas debemos hacer que esto vaya a un xarray
    #  o a por lo menos un numpy array con los datos
    #  1. Opcion: puede ser pandas luego xarray no se como eso es posible
    # return 0

    fff = get_pairwise_prob(mdl, str_event_nickname1, str_event_nickname2)
    average = db.calc_IgorBestScenarios_average_of(fff)
    print(average)

    import matplotlib.pyplot as plt
    fig, ax = plt.subplots(figsize=(10,15))
    average.plot(ax=ax)

    str_title = "$P($" + str_event_nickname1 + ", " + str_event_nickname2+"$)$"
    #ax.set_title(str_title)
    ax.set_ylabel(str_event_nickname1.replace("_", " "))
    ax.set_xlabel(str_event_nickname2.replace("_", " "))

    # plot sutilties
    import numpy as np
    v_genlabel = np.vectorize(p3.genLabel )

    XX = average[str_event_nickname2].values
    lbl_XX = average['lbl__' + str_event_nickname2].values

    YY = average[str_event_nickname1].values
    lbl_YY = average['lbl__' + str_event_nickname1].values

    ax.set_xticks(XX)
    ax.set_xticklabels(v_genlabel(lbl_XX), rotation=90)

    ax.set_yticks(YY)
    ax.set_yticklabels(v_genlabel(lbl_YY))

    fig.savefig('figura.pdf')
    # return ax

    plt.show()


if __name__ == "__main__":
    main()
